import onegov.core
import onegov.town

from onegov_testing import Client, utils


def test_view_permissions():
    utils.assert_explicit_permissions(onegov.town, onegov.town.TownApp)


def test_startpage(town_app):
    client = Client(town_app)

    links = client.get('/').pyquery('.top-bar-section a')

    assert links[0].text.strip('\n ') == 'Leben & Wohnen'
    assert links[0].attrib.get('href').endswith('/topics/leben-wohnen')

    assert links[1].text.strip('\n ') == 'Bildung & Gesellschaft'
    assert links[1].attrib.get('href').endswith('/topics/bildung-gesellschaft')

    assert links[2].text.strip('\n ') == 'Politik & Verwaltung'
    assert links[2].attrib.get('href').endswith('/topics/politik-verwaltung')

    assert links[3].text.strip('\n ') == 'Freizeit & Tourismus'
    assert links[3].attrib.get('href').endswith('/topics/freizeit-tourismus')

    assert links[4].text.strip('\n ') == 'Porträt & Wirtschaft'
    assert links[4].attrib.get('href').endswith('/topics/portraet-wirtschaft')

    links = client.get('/').pyquery('.homepage-tiles a')

    assert links[0].find('h3').text.strip('\n ') == 'Leben & Wohnen'
    assert links[0].attrib.get('href').endswith('/topics/leben-wohnen')

    assert links[1].find('h3').text.strip('\n ') == 'Bildung & Gesellschaft'
    assert links[1].attrib.get('href').endswith('/topics/bildung-gesellschaft')

    assert links[2].find('h3').text.strip('\n ') == 'Politik & Verwaltung'
    assert links[2].attrib.get('href').endswith('/topics/politik-verwaltung')

    assert links[3].find('h3').text.strip('\n ') == 'Freizeit & Tourismus'
    assert links[3].attrib.get('href').endswith('/topics/freizeit-tourismus')

    assert links[4].find('h3').text.strip('\n ') == 'Porträt & Wirtschaft'
    assert links[4].attrib.get('href').endswith('/topics/portraet-wirtschaft')


def test_view_occurrences_on_startpage(town_app):
    client = Client(town_app)
    links = [
        a.text for a in client.get('/').pyquery('.panel-links:first li a')
    ]
    events = (
        '150 Jahre Govikon',
        'Alle Veranstaltungen',
        'Gemeinsames Turnen',
        'Generalversammlung'
    )
    assert set(events) <= set(links)


def test_pages_on_homepage(es_town_app):
    client = Client(es_town_app)
    client.skip_first_form = True
    client.login_editor()

    new_page = client.get('/topics/bildung-gesellschaft').click('Thema')
    new_page.form['title'] = "0xdeadbeef"
    new_page = new_page.form.submit().follow()

    assert '0xdeadbeef' not in client.get('/')

    edit_page = new_page.click('Bearbeiten')
    edit_page.form['is_visible_on_homepage'] = True
    edit_page.form.submit()

    assert '0xdeadbeef' in client.get('/')

    edit_page = new_page.click('Bearbeiten')
    edit_page.form['is_hidden_from_public'] = True
    edit_page.form.submit()

    assert '0xdeadbeef' in client.get('/')
    assert '0xdeadbeef' not in Client(es_town_app).get('/')

    client.delete(
        new_page.pyquery('a[ic-delete-from]')[0].attrib['ic-delete-from']
    )

    assert '0xdeadbeef' not in client.get('/')



from onegov.core.utils import module_path
from onegov.org.theme import OrgTheme


# options editable by the user
user_options = {
    'primary-color': '#006fba',
    'footer-height': '38px'
}


class TownTheme(OrgTheme):
    name = 'onegov.town.foundation'

    # don't touch this number, it's incremented using bumpversion, so every
    # release will automatically trigger a rebuild of the theme
    version = '1.15.9'

    @property
    def post_imports(self):
        return super().post_imports + [
            'town'
        ]

    @property
    def extra_search_paths(self):
        base_paths = super().extra_search_paths
        return [module_path('onegov.town.theme', 'styles')] + base_paths



""" Contains upgrade tasks that are executed when the application is being
upgraded on the server. See :class:`onegov.core.upgrade.upgrade_task`.

"""
import textwrap

from onegov.core.upgrade import upgrade_task
from onegov.org.models import Organisation


@upgrade_task('Install the default homepage structure')
def install_default_homepage_structure(context):

    org = context.session.query(Organisation).first()

    if org is None:
        return

    org.meta['homepage_structure'] = textwrap.dedent("""\
        <row>
            <column span="8">
                <homepage-tiles />
                <news />
            </column>
            <column span="4">
                <panel>
                    <services />
                </panel>
                <panel>
                    <events />
                </panel>
                <panel>
                    <directories />
                </panel>
            </column>
        </row>
    """)


@upgrade_task('Install updated homepage structure')
def install_updated_homepage_structure(context):

    org = context.session.query(Organisation).first()

    if org is None:
        return

    # not a town
    if '<services />' not in org.meta['homepage_structure']:
        return

    org.meta['homepage_structure'] = textwrap.dedent("""\
        <row>
            <column span="12">
                <slider />
            </column>
        </row>
        <row>
            <column span="8">
                <row>
                    <column span="6">
                        <news />
                    </column>
                    <column span="6">
                        <events />
                    </column>
                </row>
                <line />
                <homepage-tiles />
            </column>
            <column span="4">
                <panel>
                    <services />
                </panel>
                <panel>
                    <directories />
                </panel>
            </column>
        </row>
    """)


@upgrade_task('Upgrade homepage structure to include directories')
def upgrade_homepage_structure_to_include_directories(context):
    org = context.session.query(Organisation).first()

    if org is None:
        return

    if '<services />' not in org.meta['homepage_structure']:
        return

    org.meta['homepage_structure'] = textwrap.dedent("""\
        <row>
            <column span="12">
                <slider />
            </column>
        </row>
        <row>
            <column span="8">
                <row>
                    <column span="6">
                        <news />
                    </column>
                    <column span="6">
                        <events />
                    </column>
                </row>
                <line />
                <homepage-tiles />
            </column>
            <column span="4">
                <panel>
                    <services />
                </panel>
                <panel>
                    <contacts_and_albums />
                </panel>
                <panel>
                    <directories />
                </panel>
            </column>
        </row>
    """)



""" Contains the application which builds on onegov.core and uses
more.chameleon.

It's possible that the chameleon integration is moved to onegov.core in the
future, but for now it is assumed that different applications may want to
use different templating languages.

"""

from onegov.core import utils
from onegov.org import OrgApp
from onegov.org.app import get_i18n_localedirs as get_org_i18n_localedirs
from onegov.town.theme import TownTheme
from onegov.town.initial_content import create_new_organisation


class TownApp(OrgApp):

    #: the version of this application (do not change manually!)
    version = '1.15.9'

    def configure_organisation(self, **cfg):
        cfg.setdefault('enable_user_registration', False)
        cfg.setdefault('enable_yubikey', True)
        super().configure_organisation(**cfg)


@TownApp.static_directory()
def get_static_directory():
    return 'static'


@TownApp.template_directory()
def get_template_directory():
    return 'templates'


@TownApp.setting(section='core', name='theme')
def get_theme():
    return TownTheme()


@TownApp.setting(section='i18n', name='localedirs')
def get_i18n_localedirs():
    return [utils.module_path('onegov.town', 'locale')] \
        + get_org_i18n_localedirs()


@TownApp.setting(section='org', name='create_new_organisation')
def get_create_new_organisation_factory():
    return create_new_organisation



""" The settings view, defining things like the logo or color of the org. """

from onegov.core.security import Secret
from onegov.form import Form, merge_forms, move_fields
from onegov.org import _
from onegov.org.forms import HomepageSettingsForm
from onegov.org.models import Organisation
from onegov.org.views.settings import handle_homepage_settings
from onegov.town.app import TownApp
from wtforms import BooleanField, StringField


def get_custom_settings_form(model, request):

    class CustomFieldsForm(Form):
        online_counter_label = StringField(
            label=_("Online Counter Label"),
            description=_("Forms and applications"))

        reservations_label = StringField(
            label=_("Reservations Label"),
            description=_("Daypasses and rooms"))

        daypass_label = StringField(
            label=_("SBB Daypass Label"),
            description=_("Generalabonnement for Towns"))

        publications_label = StringField(
            label=_("Publications Label"),
            description=_("Official Documents"))

        hide_publications = BooleanField(
            label=_("Hide Publications on Homepage"))

    return move_fields(
        form_class=merge_forms(HomepageSettingsForm, CustomFieldsForm),
        fields=(
            'online_counter_label',
            'reservations_label',
            'daypass_label',
            'publications_label',
            'hide_publications',
        ),
        after='homepage_image_6'
    )


@TownApp.form(model=Organisation, name='homepage-settings', template='form.pt',
              permission=Secret, form=get_custom_settings_form,
              setting=_("Homepage"), icon='fa-home', order=-900)
def custom_handle_settings(self, request, form):

    form.delete_field('homepage_cover')
    form.delete_field('homepage_structure')
    form.delete_field('redirect_homepage_to')

    return handle_homepage_settings(self, request, form)



from setuptools import setup, find_packages

name = 'onegov.town'
description = (
    'OneGov web application for small towns.'
)
version = '1.15.9'


def get_long_description():
    readme = open('README.rst').read()
    history = open('HISTORY.rst').read()

    # cut the part before the description to avoid repetition on pypi
    readme = readme[readme.index(description) + len(description):]

    return '\n'.join((readme, history))


setup(
    name=name,
    version=version,
    description=description,
    long_description=get_long_description(),
    url='http://github.com/onegov/onegov.town',
    author='Seantis GmbH',
    author_email='info@seantis.ch',
    license='GPLv2',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=name.split('.')[:-1],
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    python_requires='>=3.6',
    install_requires=[
        'onegov.org>=0.5.0',
    ],
    extras_require=dict(
        test=[
            'coverage',
            'morepath',
            'freezegun',
            'onegov_testing',
            'webtest',
            'pyquery',
            'pytest-localserver'
        ],
    ),
    entry_points="""
        [console_scripts]
        onegov-town=onegov.org.cli:cli

        [onegov]
        upgrade=onegov.town.upgrade
    """,
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
    ]
)



from onegov.core.utils import module_path
from onegov.reservation import ResourceCollection
from onegov.org.initial_content import add_builtin_forms
from onegov.org.initial_content import builtin_form_definitions
from onegov.org.initial_content import add_filesets, add_pages, load_content
from onegov.org.initial_content import add_events
from onegov.org.models import Organisation


def create_new_organisation(app, name, reply_to=None, forms=None,
                            create_files=True, path=None, locale='de_CH'):
    session = app.session()

    locales = {
        'de_CH': 'content/de.yaml',
        'fr_CH': 'content/fr.yaml',
    }

    path = path or module_path('onegov.town', locales[locale])
    content = load_content(path)

    # can only be called if no organisation is defined yet
    assert not session.query(Organisation).first()

    org = Organisation(name=name, **content['organisation'])
    org.reply_to = reply_to
    org.meta['locales'] = locale
    session.add(org)

    form_locales = {
        'de_CH': 'forms/builtin/de',
        'fr_CH': 'forms/builtin/fr',
    }

    forms = forms or builtin_form_definitions(
        module_path('onegov.town', form_locales[locale]))

    translator = app.translations.get(locale)

    def translate(text):
        return text.interpolate(translator.gettext(text))

    add_pages(session, path)
    add_builtin_forms(session, forms)
    add_resources(app.libres_context)
    add_events(session, name, translate, create_files)

    if create_files:
        add_filesets(session, name, path)

    session.flush()


def add_resources(libres_context):
    resource = ResourceCollection(libres_context)
    resource.add(
        "SBB-Tageskarte",
        'Europe/Zurich',
        type='daypass',
        name='sbb-tageskarte'
    )
